@@ -14,35 +14,18 @@
  * Thomas Leonard <tal197@ecs.soton.ac.uk>
  */
 
-#ifdef USE_GUI
-int window_handle;
-int task_handle = 0;		/* Zero means we are not yet a Wimp task */
-int *redraw_block = NULL;	/* NULL means not in a redraw loop. */
-
-int fg_colour = 0; 
-int bg_colour = 0xffffff00;
-#endif
-
+int ro_line_mode = TRUE;  /* For Ex mode we much echo chars to the screen ourselves */
 int windowed;		/* Flag - are we running inside a text window? */
 int WinLeft, WinTop;	/* We might be started inside a text window */
 int ScrollTop;		/* Make cursor movements relative to ScrollTop. */
 
-extern int r0;
-extern int r1;
-extern int r2;
-extern int r3;
-extern int r4;
-extern int r5;
-extern int r6;
-
 int old_escape_state = -1;
 int old_cursor_state = -1;
 
+#define rgb(r,g,b) ((b<<24) + (g<<16) + (r<<8))
 #define NORMAL_FG 0x00000000
 #define NORMAL_BG 0xffffffff
 
-#define rgb(r,g,b) ((b<<24) + (g<<16) + (r<<8))
-
 /* Convert a DOS colour number to an RGB palette entry.
  * Mappings from X11 rgb/txt file.
  */
@@ -76,18 +59,18 @@ map_colour(dos)
 text_fg(fg)
     int fg;		/* Foregound colour in the form &BBGGRR00 */
 {
-    swi(XOS_Bit | ColourTrans_SetTextColour, fg, 0, 0, 0);
+    xswi(ColourTrans_SetTextColour, fg, 0, 0, 0);
 }
 
     void
 text_bg(bg)
-    int bg;		/* Backgound colour in the form &BBGGRR00 */
+    int		bg;	/* Backgound colour in the form &BBGGRR00 */
 {
-    swi(XOS_Bit | ColourTrans_SetTextColour, bg, 0, 0, 1<<7);
+    xswi(ColourTrans_SetTextColour, bg, 0, 0, 1 << 7);
 }
 
 #define OUT_NORMAL 0
-#define OUT_NUMBER 1
+#define OUT_NUMBER 1		/* Reading in a number */
 
     void
 mch_write(s, len)
@@ -97,6 +80,22 @@ mch_write(s, len)
     static int mode = OUT_NORMAL;
     static int x, y;			/* For reading numbers in. */
 
+    if (!term_console)
+    {
+	/* Maybe we are running Vim remotely - don't interpret chars */
+	while (len--)
+	{
+	    char_u c = *s++;
+	    swi(OS_WriteC, c);
+	    /* We might need to send a CR too. This shouldn't
+	     * hurt if we don't need it, should it?
+	     */
+	    if (c == 10)
+		swi(OS_WriteI + 13);
+	}
+	return;
+    }
+
     while (len--)
     {
 	char_u c = *s++;
@@ -112,7 +111,6 @@ mch_write(s, len)
 		    x = (x * 10) + c - '0';
 		    continue;
 		}
-
 	    /* note: no break here! */
 
 	    case OUT_NORMAL:
@@ -211,18 +209,82 @@ mch_write(s, len)
  * Get a characters from the keyboard.
  * Return the number of characters that are available.
  * If wtime == 0 do not wait for characters.
- * If wtime == n wait n msecs for characters (not with GUI ?).
+ * If wtime == n wait n msecs for characters.
  * If wtime == -1 wait forever for characters.
  */
     int
 mch_inchar(buf, maxlen, wtime)
     char_u  *buf;
     int	    maxlen;
-    long    wtime;	    /* don't use "time", MIPS cannot handle it */
+    long    wtime;
 {
     int got=0;
     unsigned int start_time = clock();
 
+    if (ro_line_mode)
+    {
+	/* We're probably in Ex mode - get whole lines at a time. */
+
+	static char_u	line_buffer[256];
+	static int	remaining_chars = 0;
+	static int	buf_pos = 0;
+
+	/* Do we need to fetch another line? */
+	if (remaining_chars == 0)
+	{
+	    int		old_esc_state;
+	    swi(OS_Byte, 200, 1, 0xfe);
+	    old_esc_state = r1;
+
+	    buf_pos = 0;
+	    if (xswi(OS_ReadLine, line_buffer, 255, 0, 255) & (c_flag | v_flag))
+	    {
+		got_int = TRUE;	    /* ESC pressed */
+		r1 = 0;
+	    }
+	    line_buffer[r1] = 13;
+	    remaining_chars = r1 + 1;	/* Count CR as part of input */
+
+	    swi(OS_Byte, 200, old_esc_state, 0);
+	}
+
+	/* Can we send the rest of the buffer back in one go? */
+	if (remaining_chars <= maxlen)
+	{
+	    int	    got = remaining_chars;
+
+	    memcpy(buf, line_buffer + buf_pos, got);
+	    remaining_chars = 0;
+	    return  got;
+	}
+
+	/* Send as much as we can */
+	memcpy(buf, line_buffer + buf_pos, maxlen);
+	buf_pos += maxlen;
+	remaining_chars -= maxlen;
+
+	return maxlen;
+    }
+
+    if (!term_console)
+    {
+	/* Use OS_ReadC for all input.
+	 * Avoids problems with remote access getting interference from
+	 * the keyboard.
+	 */
+	if (wtime == 0)
+	    return 0;	    /* Ignore quick key checks */
+
+	if (xswi(OS_ReadC) & c_flag)
+	{
+	    got_int = TRUE;	/* ESC pressed - can this happen? */
+	    swi(OS_Byte, 124);	/* Clear Escape state */
+	    r0 = 0x1b;		/* It *might* not have been Escape! */
+	}
+	buf[0] = r0;
+	return 1;
+    }
+
     /*
      * OK, here's the plan:
      *
@@ -230,11 +292,11 @@ mch_inchar(buf, maxlen, wtime)
      * 2) Get keys until the keyboard buffer is empty or buf is full
      */
 
-    while (swi(OS_Byte,145,0) & c_flag)
+    while (xswi(OS_Byte,145,0) & c_flag)
     {
-    	/* Nothing at all in the keyboard buffer.
-    	 * Has our time expired yet?
-    	 */
+	/* Nothing at all in the keyboard buffer.
+	 * Has our time expired yet?
+	 */
 	if ( (wtime != -1) && (clock() - start_time) >= wtime )
 	    return 0;		/* Nothing read - giving up */
     }
@@ -243,10 +305,9 @@ mch_inchar(buf, maxlen, wtime)
 
     while (got < maxlen)
     {
-/* printf("{%o}",r2); */    
 	buf[got++] = r2;
 
-	if (swi(OS_Byte,145,0) & c_flag)
+	if (xswi(OS_Byte,145,0) & c_flag)
 	    return got;		/* Keyboard buffer empty */
     }
     return got;			/* buf is full */
@@ -258,7 +319,9 @@ mch_inchar(buf, maxlen, wtime)
     int
 mch_char_avail()
 {
-    if (swi(OS_Byte, 152, 0) & c_flag)
+    if (!term_console)
+	return 0;	    /* Can't tell */
+    if (xswi(OS_Byte, 152, 0) & c_flag)
 	return 0;
     return 1;
 }
@@ -274,7 +337,7 @@ mch_char_avail()
 mch_avail_mem(special)
     int special;
 {
-    if (swi(XOS_Bit | Wimp_SlotSize, -1, -1) & v_flag)
+    if (xswi(Wimp_SlotSize, -1, -1) & v_flag)
 	return 0x7fffffff;
     return r2;
 }
@@ -284,11 +347,24 @@ mch_delay(msec, ignoreinput)
     long	msec;
     int		ignoreinput;
 {
-    clock_t	end_time;
+    int		start_time, time_now;
+    int		csec = msec / 10;
 
-    end_time=clock()+(msec*CLOCKS_PER_SEC)/1000;
-    while (clock() < end_time)
-	;
+    swi(OS_ReadMonotonicTime);
+    start_time = r0;
+
+    for (;;)
+    {
+	swi(OS_ReadMonotonicTime);
+	time_now = r0;
+	if (time_now - start_time > csec)
+	    return;
+#ifdef USE_GUI
+	/* In the GUI, allow other programs to run while waiting. */
+	if (gui.in_use)
+	    gui_mch_wait_for_chars(start_time + csec);
+#endif
+    }
 }
 
 /*
@@ -298,7 +374,7 @@ mch_delay(msec, ignoreinput)
     void
 mch_suspend()
 {
-    (void)mch_call_shell(NULL, SHELL_COOKED);
+    suspend_shell();
 }
 
     void
@@ -311,12 +387,13 @@ mch_windinit()
      */
 
     int buf[7] = {132, 135, 256, 257, 1, 2, -1};
+
     /* Command windows are no longer forced open, since if we are
      * in the desktop then we'll use the GUI version.
      * Opening a command window here messes up the GUI version startup
      */
 #ifndef USE_GUI
-    swi(XOS_Bit | OS_WriteI);
+    swi(OS_WriteI);
 #endif
     swi(OS_ReadVduVariables, buf, buf);
     WinLeft = buf[0];
@@ -354,13 +431,14 @@ mch_check_win(argc, argv)
     int
 mch_input_isatty()
 {
-    if (swi(XOS_Bit | OS_ChangeRedirection, -1, -1) & v_flag)
+    if (xswi(OS_ChangeRedirection, -1, -1) & v_flag)
 	return TRUE;		/* Error - TRUE is probably correct though */
     if (r0 == 0)
 	return TRUE;
     return FALSE;
 }
 
+#ifdef WANT_TITLE
     int
 mch_can_restore_title()
 {
@@ -373,6 +451,7 @@ mch_can_restore_icon()
     return FALSE;
 }
 
+
 /*
  * Set the window title and icon.
  */
@@ -381,6 +460,20 @@ mch_settitle(title, icon)
     char_u *title;
     char_u *icon;
 {
+    if (title == NULL)
+	title = (char_u *) "<untitled>";
+#ifdef USE_GUI
+    if (gui.in_use && strcmp(title, gui.window_title))
+    {
+	int length;
+	length = strlen(title);
+	if (length >= gui.window_title_size)
+	    length = gui.window_title_size - 1;
+	strncpy(gui.window_title, title, length);
+	gui.window_title[length] = 0;
+	ro_redraw_title(gui.window_handle);
+    }
+#endif
     return;
 }
 
@@ -397,6 +490,7 @@ mch_restore_title(which)
 {
     return;
 }
+#endif
 
 /*
  * Insert user name in s[len].
@@ -420,9 +514,7 @@ mch_get_host_name(s, len)
     char_u  *s;
     int	    len;
 {
-    char_u *name;
-    int rlen;
-    if (swi(XOS_Bit | OS_ReadVarVal, "Machine$Name", s, len, 0, 3) & v_flag)
+    if (xswi(OS_ReadVarVal, "Machine$Name", s, len, 0, 3) & v_flag)
     {
 	/* Variable does not exist (normal operation) */
 	STRNCPY(s, "(unknown)", len);
@@ -435,7 +527,7 @@ mch_get_host_name(s, len)
     long
 mch_get_pid()
 {
-    if (swi(XOS_Bit | Wimp_ReadSysInfo, 5) & v_flag)
+    if (xswi(Wimp_ReadSysInfo, 5) & v_flag)
 	return 0;
     return r0;
 }
@@ -449,8 +541,8 @@ mch_dirname(buf, len)
     char_u  *buf;
     int	    len;
 {
-    if (swi(XOS_Bit | OS_FSControl, 37, "@", buf, 0, 0, len) & v_flag)
-    	return FAIL;
+    if (xswi(OS_FSControl, 37, "@", buf, 0, 0, len) & v_flag)
+	return FAIL;
     return OK;
 }
 
@@ -464,24 +556,24 @@ mch_FullName(fname, buf, len, force)
     char_u *fname, *buf;
     int len;
     int	force;		/* Also expand when already absolute path name.
-    			 * Not used under RISC OS.
-    			 */
+			 * Not used under RISC OS.
+			 */
 {
     if (fname == NULL)
-	return FAIL;    
-    if (swi(XOS_Bit | OS_FSControl, 37, fname, buf, 0, 0, len) & v_flag)
-    	return FAIL;
+	return FAIL;
+    if (xswi(OS_FSControl, 37, fname, buf, 0, 0, len) & v_flag)
+	return FAIL;
     return OK;
 }
 
 /*
- * return TRUE is fname is an absolute path name
+ * return TRUE if fname is an absolute path name
  */
     int
 mch_isFullName(fname)
     char_u	*fname;
 {
-    if (vim_strchr(fname, (char_u) '$') != NULL)
+    if (strstr(fname, "::") && strstr(fname,".$."))
 	return TRUE;
     return FALSE;
 }
@@ -534,7 +626,7 @@ mch_hide(name)
 mch_isdir(name)
     char_u *name;
 {
-    if (swi(XOS_Bit | OS_File, 17, name) & v_flag)
+    if (xswi(OS_File, 17, name) & v_flag)
 	return FALSE;
     if (r0 == 2 || r0 == 3)
 	return TRUE;		/* Count image files as directories. */
@@ -549,37 +641,48 @@ mch_windexit(r)
     exiting = TRUE;
     out_flush();
     ml_close_all(TRUE);		/* remove all memfiles */
+
 #ifdef USE_GUI
     if (gui.in_use)
 	gui_exit(r);
 #endif
-    swi(XOS_Bit | OS_NewLine);
+    swi(OS_NewLine);
     if (old_escape_state != -1)
-	swi(XOS_Bit | OS_Byte, 229, old_escape_state, 0);
+	swi(OS_Byte, 229, old_escape_state, 0);
     if (old_cursor_state != -1)
-	swi(XOS_Bit | OS_Byte, 4, old_cursor_state);
+	swi(OS_Byte, 4, old_cursor_state);
     exit(r);
 }
 
     void
 mch_settmode(tmode)
-    int		tmode;
+    int		tmode;	    /* TMODE_RAW or TMODE_COOK */
 {
-    /* Block cursor. */
-    swi(XOS_Bit | OS_WriteN,
-	"\027\000\012\000\000\000\000\000\000\000",
-	10);
+    if (tmode == TMODE_COOK)
+    {
+	ro_line_mode = TRUE;
+	return;
+    }
 
-    /* Disable the standard cursor keys actions. */
-    swi(XOS_Bit | OS_Byte, 4, 1);
-    if (old_cursor_state == -1)
-	old_cursor_state = r1;
+    ro_line_mode = FALSE;
+
+    if (term_console)
+    {
+	/* Block cursor. */
+	swi(OS_WriteN,
+		"\027\000\012\000\000\000\000\000\000\000",
+		10);
+
+	/* Disable the standard cursor key actions. */
+	swi(OS_Byte, 4, 1);
+	if (old_cursor_state == -1)
+	    old_cursor_state = r1;
+    }
 
     /* Stop Escape from quitting Vim! */
-    swi(XOS_Bit | OS_Byte, 229, 1, 0);
+    swi(OS_Byte, 229, 1, 0);
     if (old_escape_state == -1)
 	old_escape_state = r1;
-
 }
 
 /*
@@ -606,49 +709,39 @@ mch_screenmode(arg)
 /*
  * Try to get the current window size.
  * Return OK when size could be determined, FAIL otherwise.
- * Simply return results stored by mch_windinit().
+ * Simply return results stored by mch_windinit() if we are the
+ * machine's console. If not, we don't know how big the screen is.
  */
     int
 mch_get_winsize()
 {
     /* if size changed: screenalloc will allocate new screen buffers */
-    return OK;
+    return term_console ? OK : FAIL;
 }
 
     void
 mch_set_winsize()
 {
-/*    printf("mch_set_winsize \n"); */
 }
 
     int
 mch_call_shell(cmd, options)
     char_u  *cmd;
-    int	    options;	/* SHELL_FILTER if called by do_filter() */
-			/* SHELL_COOKED if term needs cooked mode */
-			/* SHELL_EXPAND if called by mch_expand_wildcards() */
+    int	    options;/* SHELL_FILTER if called by do_filter() */
+		    /* SHELL_COOKED if term needs cooked mode */
+		    /* SHELL_EXPAND if called by mch_expand_wildcards() - NOT USED */
 {
     int		retval;
 
-    if (options & SHELL_COOKED)
-	settmode(TMODE_COOK);		/* set to normal mode */
     if (cmd == NULL)
 	cmd = (char_u *) "GOS";
 
 #ifdef USE_GUI
     if (gui.in_use)
-    {
-        if (swi(Wimp_StartTask, cmd) & v_flag)
-	    retval = -1;
-        else
-        {
-	    /* [ block until finished ] */
-	    /* [ read return value ] */
-	    retval = 0;
-        }
-        return retval;
-    }
-#else
+	return gui_mch_call_shell(cmd, options);
+#endif
+    if (options & SHELL_COOKED)
+	settmode(TMODE_COOK);		/* set to normal mode */
     MSG_PUTS("\n");
 
    /* I don't even want to think about what UnixLib must
@@ -659,18 +752,18 @@ mch_call_shell(cmd, options)
 	EMSG(strerror(EOPSYS));		/* Doesn't seem to set errno? */
 
     swi(OS_Byte, 229, 1, 0);		/* Re-disable escape */
-#endif
     settmode(TMODE_RAW);		/* set to raw mode */
-    return retval;
+    return retval ? FAIL : OK;
 }
 
 /*
  * Check for Escape being pressed right now.
+ * [ different if !term_console? ]
  */
     void
 mch_breakcheck()
 {
-    if (swi(OS_Byte, 121, 0xf0) & v_flag)
+    if (xswi(OS_Byte, 121, 0xf0) & v_flag)
 	return;
     if (r1 == 0xff)
     {
@@ -682,6 +775,7 @@ mch_breakcheck()
 /*
  * Recursively expand one path component into all matching files and/or
  * directories.
+ * "path" has backslashes before chars that are not to be expanded.
  * Return the number of matches found.
  */
     int
@@ -822,7 +916,7 @@ expand_section(gap, root, rest, flags)
     while (dir_pos != -1)
     {
 	buf[buflen] = 0;
-	if (swi(XOS_Bit | OS_GBPB, 9,
+	if (xswi(OS_GBPB, 9,
 		buf,				/* Directory to scan. */
 		buf + buflen + (1 - colon),	/* Buffer for result. */
 		1,			/* Number of objects to read. */
@@ -870,7 +964,7 @@ expand_section(gap, root, rest, flags)
 
 /*
  * mch_expand_wildcards() - this code does wild-card pattern matching using
- * the shell
+ * the shell. It isn't used under RISC OS.
  *
  * return OK for success, FAIL for error (you may lose some memory) and put
  * an error message in *file.
@@ -905,23 +999,29 @@ mch_has_wildcard(p)
 mch_remove(file)
     char_u *file;	/* Name of file to delete. */
 {
-    if (swi(XOS_Bit | OS_FSControl, 27, file, 0, 0) & v_flag)
+    if (xswi(OS_FSControl, 27, file, 0, 0) & v_flag)
 	return EXIT_FAILURE;
     return EXIT_SUCCESS;
 }
 
 /* Try to make existing scripts work without modification.
+ * Return a pointer to the new string (freed by caller), or NULL
  *
  * Two main cases:
  * - Absolute : $VIM/syntax/help.vim
  * - Relative : Adfs::4.$.!Vim.Resources.Syntax/help.vim
  */
-    void
+    char_u *
 mch_munge_fname(fname)
     char_u *fname;
 {
     char_u c;
     int len;
+    char_u *retval;
+
+    retval = fname = vim_strsave(fname);
+    if (fname == NULL)
+	return NULL;
 
     if (strncmp(fname, "$VIM/", 5) == 0)
     {
@@ -939,6 +1039,12 @@ mch_munge_fname(fname)
     }
     else
     {
+	/* Check to see if the file exists without modification. */
+	if (xswi(OS_File, 17, fname) & v_flag)
+	    r0 == 0;		/* Invalid filename? */
+	if (r0)
+	    return retval;
+
 	len = strlen(fname);
 	if (strcmp(fname + len - 4, ".vim") == 0)
 	{
@@ -950,739 +1056,150 @@ mch_munge_fname(fname)
 	    }
 	}
     }
-}
-
-
-/* GUI specific stuff. */
-
-#ifdef USE_GUI
-/*
- * Parse the GUI related command-line arguments.  Any arguments used are
- * deleted from argv, and *argc is decremented accordingly.  This is called
- * when vim is started, whether or not the GUI has been started.
- */
-    void
-gui_mch_prepare(int *argc, char **argv)
-{
-}
-
-/* Fatal error on initialisation - report it and die. */
-void ro_die(char_u *error)
-{
-    swi(Wimp_ReportError, error, 5, "GVim");
-    exit(EXIT_FAILURE);
-}
-
-/*
- * Initialise the RISC OS GUI.
- * Create all the windows.
- * Returns OK for success, FAIL when the GUI can't be started.
- */
-    int
-gui_mch_init()
-{
-    int name[3];
-    char *window;
-    char *data;
-
-    if (swi(XOS_Bit | Wimp_Initialise, 310, 0x4b534154, "GVim", 0) & v_flag)
-	return FAIL;
-    task_handle = r1;
-    if (swi(XOS_Bit | Wimp_OpenTemplate, 0, "Vim:Templates") & v_flag)
-	ro_die( (char *) r0);
-    strcpy( (char *) name, "editor");
-    if (swi(XOS_Bit | Wimp_LoadTemplate, 0, 0, 0, 0, -1, name, 0) & v_flag)
-	ro_die( (char *) r0);
-    window = malloc(r1);	/* Don't print text messages from alloc() */
-    data = malloc(r2);
-    if (window == NULL || data == NULL)
-	ro_die("\0\0\0\0Out of memory - Can't load templates");
-    swi(Wimp_LoadTemplate, 0,
-				window,		/* Temp block */
-				data,		/* Icon data */
-				data + r2 + 1,	/* End of icon data */
-				-1,		/* No fonts */
-				name, 0);	/* First match */
-    if (r6 == 0)
-	ro_die("\0\0\0\0Can't find window in Templates file");
-    if (swi(XOS_Bit | Wimp_CreateWindow, 0, window) & v_flag)
-	ro_die( (char *) r0);
-    window_handle = r0;
-    vim_free(window);
-
-    /* Set default foreground and background colours. */
-    gui.norm_pixel = gui.def_norm_pixel;
-    gui.back_pixel = gui.def_back_pixel;
-
-    /* Get the colors from the "Normal" and "Menu" group (set in syntax.c or
-     * in a vimrc file) */
-    set_normal_colors();
-
-    /*
-     * Check that none of the colors are the same as the background color
-     */
-    gui_check_colors();
-
-    /* [ set geometry ] */
-    return OK;
-}
-
-/*
- * Called when the foreground or background color has been changed.
- */
-    void
-gui_mch_new_colors()
-{
-}
-
-/*
- * Open the GUI window which was created by a call to gui_mch_init().
- */
-    int
-gui_mch_open(void)
-{
-    int block[8];
-    block[0] = window_handle;
-    swi(Wimp_GetWindowState, 0, block);
-    block[7] = -1;
-    swi(Wimp_OpenWindow, 0, block);
-    gui.num_cols = (block[3] - block[1]) / gui.char_width;
-    gui.num_rows = (block[4] - block[2]) / gui.char_height;
-    swi(Wimp_SetCaretPosition, window_handle, -1, 0, 0, -1, -1);
-    return OK;
-}
-
-    void
-gui_mch_exit(int rc)
-{
-    exit(rc);
-}
-
-    void
-gui_mch_set_winsize(width, height,
-			min_width, min_height,
-			base_width, base_height)
-    int width;
-    int height;
-    int min_width;
-    int min_height;
-    int base_width;
-    int base_height;
-{
-}
-
-/*
- * Allow 10 pixels for horizontal borders, 30 for vertical borders.
- * Is there no way in X to find out how wide the borders really are?
- */
-    void
-gui_mch_get_screen_dimensions(int *screen_w, int *screen_h)
-{
-}
-
-/*
- * Initialise vim to use the font with the given name.
- * Return FAIL if the font could not be loaded, OK otherwise.
- */
-    int
-gui_mch_init_font(char_u *font_name)
-{
-    return OK;
-}
-
-/*
- * Get a font structure for highlighting.
- */
-    GuiFont
-gui_mch_get_font(char_u *name, int giveErrorIfMissing)
-{
-}
-
-/*
- * Set the current text font.
- */
-    void
-gui_mch_set_font(GuiFont font)
-{
-    gui.char_width = 16;
-    gui.char_height = 32;
-}
-
-/*
- * Return TRUE if the two fonts given are equivalent.
- */
-    int
-gui_mch_same_font(GuiFont f1, GuiFont f2)
-{
-    return FALSE;
-}
-
-/*
- * If a font is not going to be used, free its structure.
- */
-    void
-gui_mch_free_font(GuiFont font)
-{
-}
-
-/*
- * Return the Pixel value(color) for the given color name.
- * Return -1 for error.
- */
-    GuiColor
-gui_mch_get_color(char_u *name)
-{
-    int		i;
-    struct colour
-    {
-    	char_u	*name;
-    	int	value;
-    } colours[] =
-    {
-	{ "Black",		rgb(0,		0,	0)	},
-	{ "DarkBlue",		rgb(0,		0,	139)	},
-	{ "DarkGreen",		rgb(0,		100,	0)	},
-	{ "DarkCyan",		rgb(0,		139,	139)	},
-	{ "DarkRed",		rgb(139,	0,	0)	},
-	{ "DarkMagenta",	rgb(139,	0,	139)	},
-	{ "Brown",		rgb(165,	42,	42)	},
-	{ "LightGray",		rgb(211,	211,	211)	},
-	{ "LightGrey",		rgb(211,	211,	211)	},
-	{ "Gray",		rgb(211,	211,	211)	},
-	{ "Grey",		rgb(211,	211,	211)	},
-	{ "DarkGray",		rgb(169,	169,	169)	},
-	{ "DarkGrey",		rgb(169,	169,	169)	},
-	{ "Blue",		rgb(173,	216,	230)	},
-	{ "LightBlue",		rgb(173,	216,	230)	},
-	{ "Green",		rgb(144,	238,	144)	},
-	{ "LightGreen",		rgb(144,	238,	144)	},
-	{ "Cyan",		rgb(224,	255,	255)	},
-	{ "LightCyan",		rgb(224,	255,	255)	},
-	{ "Red",		rgb(255,	0,	0)	},
-	{ "LightRed",		rgb(255,	0,	0)	},
-	{ "Magenta",		rgb(255,	0,	255)	},
-	{ "LightMagenta",	rgb(255,	0,	255)	},
-	{ "Yellow",		rgb(255,	255,	0)	},
-	{ "White",		rgb(255,	255,	255)	},
-	{NULL, 0}
-    };
-
-    for (i = 0; ; i++)
-    {
-	if (colours[i].name == NULL)
-	    return (GuiColor) -1;
-	if (STRICMP(name, colours[i].name) == 0)
-	    return colours[i].value;
-    }
-    printf("[ missing colour %s ]\n", name);
-    return (GuiColor) -1;
-}
-
-/*
- * Set the current text foreground color.
- */
-    void
-gui_mch_set_fg_color(GuiColor colour)
-{
-    swi(ColourTrans_ReturnGCOL, colour);
-    fg_colour = r0;
-}
-
-/*
- * Set the current text background color.
- */
-    void
-gui_mch_set_bg_color(GuiColor colour)
-{
-    swi(ColourTrans_ReturnGCOL, colour);
-    bg_colour = r0;
-}
-
-    void
-gui_mch_draw_string(int row, int col, char_u *s, int len, int flags)
-{
-    int x, y;		/* Workarea x,y */
-
-    x = col * gui.char_width;
-    y = -row * gui.char_height;
-
-    if (redraw_block)
-    {
-	swi(ColourTrans_SetColour, fg_colour, 0, 0, 0, 0);
-	swi(ColourTrans_SetColour, bg_colour, 0, 0, 1 << 7, 0);
-	swi(OS_Plot, 4,			/* Move the drawing cursor */
-		x + redraw_block[1],
-		y + redraw_block[4] - redraw_block[6] - 1);
-	swi(OS_WriteN, s, len);
-    }
-    else
-    {
-        int block[44];
-        block[0] = window_handle;
-        block[1] = x;
-        block[2] = y - gui.char_height;
-        block[3] = (col + len) * gui.char_width;
-        block[4] = y;
-        swi(Wimp_UpdateWindow, 0, block);
-        while (r0)
-        {
-	    swi(ColourTrans_SetColour, fg_colour, 0, 0, 0, 0);
-	    swi(ColourTrans_SetColour, bg_colour, 0, 0, 1 << 7, 0);
-	    swi(OS_WriteI + 16);
-	    swi(OS_Plot, 4,
-	    		x + block[1],
-	    		y + block[4] - block[6] - 1);
-	    swi(OS_WriteN, s, len);
-	    swi(Wimp_GetRectangle, 0, block);
-        }
-    }
-}
-
-/*
- * Return OK if the key with the termcap name "name" is supported.
- */
-    int
-gui_mch_haskey(char_u *name)
-{
-    return FAIL;
-}
-
-    void
-gui_mch_beep(void)
-{
-    swi(XOS_Bit | OS_WriteI + 7);
-}
-
-/*
- * Visual bell.
- */
-    void
-gui_mch_flash(void)
-{
-}
-
-/*
- * Invert a rectangle from row r, column c, for nr rows and nc columns.
- */
-    void
-gui_mch_invert_rectangle(int r, int c, int nr, int nc)
-{
-}
-
-/*
- * Iconify the GUI window.
- */
-    void
-gui_mch_iconify(void)
-{
-}
-
-/*
- * Draw a cursor without focus.
- */
-    void
-gui_mch_draw_hollow_cursor(GuiColor color)
-{
-}
-
-/*
- * Draw part of a cursor, "w" pixels wide, and "h" pixels high, using
- * color "color".
- */
-    void
-gui_mch_draw_part_cursor(int w, int h, GuiColor color)
-{
-}
-
-/*
- * Catch up with any queued events.  This may put keyboard input into the
- * input buffer, call resize call-backs, trigger timers etc.  If there is
- * nothing in the event queue(& no timers pending), then we return
- * immediately.
- */
-    void
-gui_mch_update(void)
-{
-    int block[64];
-    swi(Wimp_Poll, 0, block);
-    if (r0)
-    	process_event(r0, block);
-}
-
-    void
-redraw_window(block)
-    int *block;
-{
-    int x, y;			/* Vim workarea coords */
-
-    swi(Wimp_RedrawWindow, 0, block);
-    redraw_block = block;
-    while (r0)
-    {
-    	swi(OS_WriteI + 16);
-    	x = block[7] - block[1];
-    	y = block[4] - block[10] - block[6];
-    	gui_redraw(x , y, block[9] - block[7], block[10] - block[8]);
-    	swi(Wimp_GetRectangle, 0, block);
-    }
-    redraw_block = NULL;
-}
-
-    void
-process_event(event, block)
-    int event;
-    int *block;
-{
-    switch (event)
-    {
-	case 1:
-	    redraw_window(block);
-	    break;
-	case 2:
-	    swi(Wimp_OpenWindow, 0, block);
-	    break;
-	case 3:
-	    gui_exit(1);
-	    break;
-	case 6:		/* Mouse click. */
-	    swi(Wimp_SetCaretPosition, window_handle, -1, 0, 0, -1, -1);
-	    break;
-	case 8:
-	    add_to_input_buf(&block[6], 1);
-	    break;
-	case 17:
-	case 18:
-	    if (block[4] == 0)
-	        exit(1);
-	    break;
-    }
-}
-
-/*
- * GUI input routine called by gui_wait_for_chars().  Waits for a character
- * from the keyboard.
- *  wtime == -1	    Wait forever.
- *  wtime == 0	    This should never happen.
- *  wtime > 0	    Wait wtime milliseconds for a character.
- * Returns OK if a character was found to be available within the given time,
- * or FAIL otherwise.
- */
-    int
-gui_mch_wait_for_chars(long wtime)
-{
-    int block[64];
-    if (wtime == -1)
-    	swi(Wimp_Poll, 1, block);
-    else
-    {
-	swi(OS_ReadMonotonicTime);
-	swi(Wimp_PollIdle, 0, block, r0 + (wtime / 10));
-    }
-    if (r0)
-	process_event(r0, block);
-    return vim_is_input_buf_empty() ? FAIL : OK;
-}
-
-/* Flush any output to the screen */
-    void
-gui_mch_flush(void)
-{
-}
-
-/*
- * Clear a rectangular region of the screen from text pos(row1, col1) to
- *(row2, col2) inclusive.
- */
-    void
-gui_mch_clear_block(int row1, int col1, int row2, int col2)
-{
-    if (redraw_block)
-    {
-	swi(OS_Plot, 4, TEXT_X(col1), -TEXT_Y(row2 + 1));
-	swi(OS_Plot, 96 + 7, TEXT_X(col2 + 1), -TEXT_Y(row1));
-    }
-    else
-    {
-	int block[44];
-	block[0] = window_handle;
-	block[1] = TEXT_X(col1);
-	block[2] = -TEXT_Y(row2 + 1);
-	block[3] = TEXT_X(col2 + 1);
-	block[4] = -TEXT_Y(row1);
-	swi(Wimp_UpdateWindow, 0, block);
-	while (r0)
-	{
-	    swi(OS_WriteI + 16);
-	    swi(Wimp_GetRectangle, 0, block);
-	}
-    }
-}
-
-    void
-gui_mch_clear_all(void)
-{
-    if (redraw_block)
-    {
-	swi(OS_WriteI + 16);
-    }
-    else
-    {
-	int block[44];
-	block[0] = window_handle;
-	block[1] = 0;
-	block[2] = -gui.num_rows * gui.char_height;
-	block[3] = gui.num_cols * gui.char_width;
-	block[4] = 0;
-	swi(Wimp_UpdateWindow, 0, block);
-	while (r0)
-	{
-	    swi(OS_WriteI + 16);
-	    swi(Wimp_GetRectangle, 0, block);
-        }
-    }
-}
-
-/*
- * Delete the given number of lines from the given row, scrolling up any
- * text further down within the scroll region.
- */
-    void
-gui_mch_delete_lines(int row, int num_lines)
-{
-    int top_from = -row - num_lines;
-    int bot_from = -gui.scroll_region_bot - 1;
-    int bot_to   = bot_from + num_lines;
-
-    swi(Wimp_BlockCopy, window_handle,
-    			0,
-    			bot_from * gui.char_height,
-    			gui.num_cols * gui.char_width,
-    			top_from * gui.char_height,
-
-    			0,
-    			bot_to * gui.char_height);
-    gui_mch_clear_block(gui.scroll_region_bot - num_lines + 1,
-    			0,
-    			gui.scroll_region_bot,
-    			gui.num_cols - 1);
-}
-
-/*
- * Insert the given number of lines before the given row, scrolling down any
- * following text within the scroll region.
- */
-    void
-gui_mch_insert_lines(int row, int num_lines)
-{
-    int top_from = -row;
-    int bot_to   = -gui.scroll_region_bot - 1;
-    int bot_from = bot_to + num_lines;
-
-    swi(Wimp_BlockCopy, window_handle,
-    			0,
-    			bot_from * gui.char_height,
-    			gui.num_cols * gui.char_width,
-    			top_from * gui.char_height,
-
-    			0,
-    			bot_to * gui.char_height);
-    gui_mch_clear_block(row,
-    			0,
-    			row + num_lines - 1,
-    			gui.num_cols - 1);
-}
-
-    void
-clip_mch_request_selection(void)
-{
-}
-
-    void
-clip_mch_lose_selection(void)
-{
-}
-
-    int
-clip_mch_own_selection(void)
-{
-    return FALSE;
-}
-
-/*
- * Send the current selection to the clipboard.  Do nothing for X because we
- * will fill in the selection only when requested by another app.
- */
-    void
-clip_mch_set_selection(void)
-{
-}
-
-/*
- * Make a menu either grey or not grey.
- */
-    void
-gui_mch_menu_grey(GuiMenu *menu, int grey)
-{
-}
-
-/*
- * Make menu item hidden or not hidden
- */
-    void
-gui_mch_menu_hidden(GuiMenu *menu, int hidden)
-{
-}
-
-/*
- * This is called after setting all the menus to grey/hidden or not.
- */
-    void
-gui_mch_draw_menubar(void)
-{
-}
-
-    void
-gui_mch_enable_scrollbar(GuiScrollbar *sb, int flag)
-{
-}
-
-    void
-gui_mch_set_blinking(long waittime, long on, long off)
-{
-}
-
-/*
- * Stop the cursor blinking.  Show the cursor if it wasn't shown.
- */
-    void
-gui_mch_stop_blink(void)
-{
-}
-
-/*
- * Start the cursor blinking.  If it was already blinking, this restarts the
- * waiting time and shows the cursor.
- */
-    void
-gui_mch_start_blink(void)
-{
-}
-
-/*
- * Return the lightness of a pixel.  White is 255.
- * Uses CIE luminance weights as given in the ChangeFSI help file.
- */
-    int
-gui_mch_get_lightness(GuiColor pixel)
-{
-    int red   = (pixel >> 24) & 0xff;
-    int green = (pixel >> 16) & 0xff;
-    int blue  = (pixel >> 8 ) & 0xff;
-    return ((red * 299) + (green * 587) + (blue * 114)) / 1000;
-}
-
-/*
- * Return the RGB value of a pixel as "#RRGGBB".
- */
-    char_u *
-gui_mch_get_rgb(GuiColor pixel)
-{
-    static char_u retval[10];
-    sprintf((char *)retval, "#%02x%02x%02x",
-	    (unsigned) ((pixel >> 24) & 0xff),
-	    (unsigned) ((pixel >> 16) & 0xff),
-	    (unsigned) ((pixel >> 8)  & 0xff));
     return retval;
 }
 
-    void
-gui_mch_set_text_area_pos(int x, int y, int w, int h)
-{
-}
-
-    void
-gui_mch_enable_menu(int flag)
-{
-}
-
-    void
-gui_mch_set_menu_pos(int x, int y, int w, int h)
-{
-}
-
-    void
-gui_mch_add_menu(GuiMenu *menu, GuiMenu *parent)
-{
-}
-
-    void
-gui_mch_add_menu_item(GuiMenu *menu, GuiMenu *parent)
-{
-}
-
-    void
-gui_mch_new_menu_colors(void)
-{
-}
-
-    void
-gui_mch_destroy_menu(GuiMenu *menu)
-{
-}
-
-    void
-gui_mch_set_scrollbar_thumb(GuiScrollbar *sb, int val, int size, int max)
-{
-}
-
-    void
-gui_mch_set_scrollbar_pos(GuiScrollbar *sb, int x, int y, int w, int h)
-{
-}
-
-    void
-gui_mch_create_scrollbar(GuiScrollbar *sb, int orient)
-{
-}
-
-    void
-gui_mch_destroy_scrollbar(GuiScrollbar *sb)
-{
-}
-
-    void
-gui_mch_set_scrollbar_colors(GuiScrollbar *sb)
-{
-}
-
-/*
- * Get current y mouse coordinate in text window.
- * Return -1 when unknown.
+/* QuickFix reads munged names from the error file.
+ * Correct them.
  */
     int
-gui_mch_get_mouse_y()
+ro_buflist_add(old_name)
+    char_u  *old_name;	/* Name of file found by quickfix */
 {
-    return -1;
+    char_u  *fname;
+    char_u  *leaf;	/* Pointer to start of leaf in old_name */
+    char_u  *ptr;
+    char_u  c;
+    int	    retval;
+
+    if (old_name == NULL)
+	return buflist_add(NULL);
+
+    /* Copy the name so we can mess around with it. */
+    fname = vim_strsave(old_name);
+    if (fname == NULL)
+	return buflist_add(old_name);    /* Out of memory - can't modify name */
+
+    /* Change `dir/main.c' into `dir.c.main' */
+    leaf = fname;
+    for (ptr = fname; c = *ptr; ptr++)
+    {
+	if (c == '/')
+	{
+	    leaf = ptr + 1;
+	    *ptr = '.';
+	}
+	else if (c == '.')
+	    break;
+    }
+    if (c == '.')
+    {
+	/* Change `main.c' into `c.main'
+	 *	  |    |
+	 *      leaf  ptr
+	 */
+	ptr += old_name - fname;
+	*ptr = '\0';
+	sprintf(leaf,
+		"%s.%s",
+		ptr + 1,
+		leaf - fname + old_name);
+    }
+
+    retval = buflist_add(fname);
+    free(fname);
+    return retval;
 }
 
-/* MouseTo(x, y) */
-    void
-gui_mch_setmouse(x, y)
-    int		x;
-    int		y;
+/* Change the current directory.
+ * Strip trailing dots to make it easier to use with filename completion.
+ */
+    int
+mch_chdir(dir)
+    char_u  *dir;
 {
+    int	    length;
+    int	    retval;
+    char_u  *new_dir;
+
+    length = strlen(dir);
+    if (dir[length - 1] != '.')
+	return chdir(dir);	    /* No trailing dots - nothing to do. */
+    new_dir = vim_strsave(dir);
+    if (new_dir == NULL)
+	return chdir(dir);	    /* Can't allocate memory. */
+
+    while (new_dir[--length] == '.')
+	new_dir[length] = '\0';
+
+    retval = chdir(new_dir);
+    vim_free(new_dir);
+    return retval;
+}
+
+/* Examine the named file, and set the 'osfiletype' option
+ * (in curbuf) to the file's type.
+ */
+    void
+mch_read_filetype(file)
+    char_u  *file;
+{
+    int	    type;
+    char_u  type_string[9];
+    int	    i;
+
+    if (xswi(OS_File, 23, file) & v_flag)
+	type = 0xfff;		/* Default to Text */
+    else
+	type = r6;
+
+    /* Type is the numerical value - see if we have a textual equivalent */
+    swi(OS_FSControl, 18, 0, type);
+    ((int *) type_string)[0] = r2;
+    ((int *) type_string)[1] = r3;
+    type_string[8] = 0;
+    for (i = 0; type_string[i] > ' '; i++)
+	;
+    type_string[i] = 0;
+
+    set_string_option_direct("osfiletype", -1, type_string, TRUE);
+    return;
 }
 
     void
-gui_mch_show_popupmenu(GuiMenu *menu)
+mch_set_filetype(file, type)
+    char_u  *file;
+    char_u  *type;
 {
-    /* TODO */
+    if (xswi(OS_FSControl, 31, type) & v_flag)
+    {
+	EMSG("Invalid 'filetype' option - using Text");
+	r2 = 0xfff;
+    }
+
+    swi(OS_File, 18, file, r2);
 }
 
-    void
-gui_mch_toggle_tearoffs(enable)
-    int		enable;
+/* Return TRUE if the file's type matches 'type'
+ * RISC OS types always start with '&'
+ */
+    int
+mch_check_filetype(fname, type)
+    char_u  *fname;
+    char_u  *type;
 {
-    /* no tearoff menus */
+    int	    value;
+    char    *end;
+
+    if (*type != '&')
+	return FALSE;
+
+    value = strtol(type + 1, &end, 16);
+    if (*end)
+	return FALSE;		/* Invalid type (report error?) */
+
+    if (xswi(OS_File, 23, fname) & v_flag)
+	return FALSE;		/* Invalid filename? */
+
+    return (r0 && r6 == value);
 }
-#endif
