@@ -22,7 +22,7 @@
 #define EFN	0			/* Event flag */
 
 #ifdef PROTO
-# define DESC int
+# typedef int DESC
 #endif
 
 static void	vms_flushbuf(void);
@@ -84,13 +84,20 @@ Window		x11_window = 0;
 Display		*x11_display = NULL;
 int		got_x_error = FALSE;
 
+# ifdef WANT_TITLE
 static int	get_x11_windis __ARGS((void));
-static void set_x11_title __ARGS((char_u *));
-static void set_x11_icon __ARGS((char_u *));
+static void	set_x11_title __ARGS((char_u *));
+static void	set_x11_icon __ARGS((char_u *));
+# endif
 #endif
 
+#ifdef WANT_TITLE
 static int	get_x11_title __ARGS((int));
 static int	get_x11_icon __ARGS((int));
+static char_u	*oldtitle = NULL;
+static char_u	*fixedtitle = (char_u *)"Thanks for flying Vim";
+static char_u	*oldicon = NULL;
+#endif
 
 static void	may_core_dump __ARGS((void));
 
@@ -106,9 +113,6 @@ static int	have_wildcard __ARGS((int, char_u **));
 static int	have_dollars __ARGS((int, char_u **));
 
 static int	do_resize = FALSE;
-static char_u	*oldtitle = NULL;
-static char_u	*fixedtitle = (char_u *)"Thanks for flying Vim";
-static char_u	*oldicon = NULL;
 static char_u	*extra_shell_arg = NULL;
 static int	show_shell_mess = TRUE;
 static int	core_dump = FALSE;	/* core dump in mch_windexit() */
@@ -421,10 +425,11 @@ mch_suspend(void)
 #endif
 #ifdef SIGTSTP
     vms_flushbuf();	/* needed to make cursor visible on some systems */
-    settmode(0);
+    settmode(TMODE_COOK);
     vms_flushbuf();	/* needed to disable mouse on some systems */
     kill(0, SIGTSTP);	/* send ourselves a STOP signal */
 
+# ifdef WANT_TITLE
     /*
      * Set oldtitle to NULL, so the current title is obtained again.
      */
@@ -433,19 +438,20 @@ mch_suspend(void)
 	vim_free(oldtitle);
 	oldtitle = NULL;
     }
-    settmode(1);
+# endif
+    settmode(TMODE_RAW);
 #else
     if (!sw_detached)
     {
-	OUT_STR("new shell started\n");
-	(void)mch_call_shell(NULL, SHELL_COOKED);
+	suspend_shell();
 	return;
     }
     vms_flushbuf();
-    settmode(0);				/* set to cooked mode */
+    settmode(TMODE_COOK);		/* set to cooked mode */
 
     sprintf(symstr, "%08X", pid);
-    mch_setenv("VIMPID", symstr);
+    /* added 3rd parameter SK 980904 */
+    mch_setenv("VIMPID", symstr, 1);
 
     /* windgoto(wins[cw].rows-1,0);	*/
     /* putchar('\r');	*/
@@ -456,16 +462,18 @@ mch_suspend(void)
     res = lib$attach(&ppid);
     if (!(res & 1))
     {
-	OUT_STR("\nDetach failed, new shell started\n");
-	(void)mch_call_shell(NULL, SHELL_COOKED);
+	OUT_STR("\nDetach failed");
+	suspend_shell();
     }
 
     /* set_tty(P(P_LI), P(P_CO));*/
     /* screenclear(cw);*/
     /* updatescreen();*/
 
-    settmode(1);					/* set to raw mode */
+    settmode(TMODE_RAW);		/* set to raw mode */
+#ifdef WANT_TITLE
     resettitle();
+#endif
 #endif
     need_check_timestamps = TRUE;
 }
@@ -528,6 +536,8 @@ mch_check_input(void)
 	return FAIL;
 }
 
+#ifdef WANT_TITLE
+
 #if defined(HAVE_X11) && defined(WANT_X11)
 
 /*
@@ -585,7 +595,8 @@ get_x11_windis(void)
 	    x11_display = NULL;
 	    x11_display_opened_here = FALSE;
 	}
-	return gui_get_x11_windis(&x11_window, &x11_display);
+	gui_get_x11_windis(&x11_window, &x11_display);
+	return OK;
     }
 #endif
 
@@ -889,6 +900,21 @@ mch_settitle(char_u *title, char_u *icon)
     }
 }
 
+/*
+ * Restore the window/icon title.
+ * which is one of:
+ * 1  Just restore title
+ * 2  Just restore icon
+ * 3  Restore title and icon
+ */
+    void
+mch_restore_title(int which)
+{
+    mch_settitle((which & 1) ? oldtitle : NULL, (which & 2) ? oldicon : NULL);
+}
+
+#endif /* WANT_TITLE */
+
     int
 vim_is_xterm(char_u *name)
 {
@@ -922,19 +948,6 @@ is_fastterm(char_u *name)
 	    vim_strnicmp((char *)name, "sun-cmd", (size_t)7) == 0);
 }
 
-/*
- * Restore the window/icon title.
- * which is one of:
- *	1  Just restore title
- *  2  Just restore icon
- *	3  Restore title and icon
- */
-    void
-mch_restore_title(int which)
-{
-    mch_settitle((which & 1) ? oldtitle : NULL, (which & 2) ? oldicon : NULL);
-}
-
 /*
  * Insert user name in s[len].
  * Return OK if a name found.
@@ -1132,8 +1145,10 @@ mch_isdir(char_u *name)
     void
 mch_windexit(int r)
 {
+#ifdef WANT_TITLE
     mch_settitle(oldtitle, oldicon);	/* restore xterm title */
-    settmode(0);
+#endif
+    settmode(TMODE_COOK);
     exiting = TRUE;
     stoptermcap();
     vms_flushbuf();
@@ -1425,8 +1440,7 @@ mch_set_winsize(void)
 /*
  *	options		SHELL_FILTER	if called by do_filter()
  *			SHELL_COOKED	if term needs cooked mode
- *			SHELL_EXPAND	if called by
- *					mch_expand_wildcards()
+ *			SHELL_EXPAND	if called by mch_expand_wildcards()
  */
 
     int
@@ -1437,7 +1451,7 @@ mch_call_shell(char_u *cmd, int options)
 
     vms_flushbuf();
     if (options & SHELL_COOKED)
-	settmode(0);				/* set to cooked mode */
+	settmode(TMODE_COOK);			/* set to cooked mode */
     if (cmd)
     {
 	if (ofn = strchr((char *)cmd, '>'))
@@ -1466,8 +1480,10 @@ mch_call_shell(char_u *cmd, int options)
 	msg_outnum((long)x);
 	OUT_STR(" returned\n");
     }
-    settmode(1);					/* set to raw mode */
+    settmode(TMODE_RAW);			/* set to raw mode */
+#ifdef WANT_TITLE
     resettitle();
+#endif
     return x;
 }
 
@@ -1512,9 +1528,11 @@ mch_breakcheck(void)
     int
 WaitForChar(long msec)
 {
-    if (vim_is_input_buf_empty()) {
-	if(RealWaitForChar(0,msec)) {
-	    add_to_input_buf((char_u *)ibuf,1);
+    if (vim_is_input_buf_empty())
+    {
+	if (RealWaitForChar(0, msec))
+	{
+	    add_to_input_buf((char_u *)ibuf, 1);
 	    return 1;
 	}
 	else
@@ -1559,8 +1577,8 @@ char_u **vms_fmatch = NULL;
  * We want to save each match for later retrieval.
  *
  * Returns:  1 - continue finding matches
- *           0 - stop trying to find any further mathces
- * 
+ *	     0 - stop trying to find any further mathces
+ *
  */
 int
 vms_wproc( char *name, int type )
@@ -1618,11 +1636,11 @@ vms_wproc( char *name, int type )
 	vms_fmatch[vms_match_num] = vim_strsave((char_u *)xname);
     }
     else {
-	vms_fmatch[vms_match_num] = 
+	vms_fmatch[vms_match_num] =
 	    vim_strsave((char_u *)xname);
     }
 #else
-    vms_fmatch[vms_match_num] = 
+    vms_fmatch[vms_match_num] =
 	vim_strsave((char_u *)xname);
 #endif
     ++vms_match_num;
@@ -1657,11 +1675,13 @@ mch_expand_wildcards(int num_pat, char_u **pat, int *num_file, char_u ***file, i
     files_alloced = EXPL_ALLOC_INC;
     files_free = EXPL_ALLOC_INC;
     *file = (char_u **) alloc(sizeof(char_u **) * files_alloced);
-    if (*file == NULL ){
+    if (*file == NULL )
+    {
 	*num_file = 0;
 	return FAIL;
     }
-    for (i=0; i<num_pat; i++) {
+    for (i = 0; i < num_pat; i++)
+    {
 	/* expand environment var or home dir */
 	if (vim_strchr(pat[i],'$') || vim_strchr(pat[i],'~'))
 	    expand_env(pat[i],buf,MAXPATHL);
@@ -1686,7 +1706,7 @@ mch_expand_wildcards(int num_pat, char_u **pat, int *num_file, char_u ***file, i
 	    /* allocate memory for pointers */
 	    if(--files_free < 1 ) {
 		files_alloced += EXPL_ALLOC_INC;
-		*file = (char_u **)realloc(*file, 
+		*file = (char_u **)realloc(*file,
 		    sizeof(char_u **) * files_alloced);
 		if( *file == NULL ) {
 		    *file = (char_u **)"";
@@ -1868,7 +1888,7 @@ vms_sys(char *cmd, char *log, char *inp)
     if (inp)
 	vul_desc(&idsc, inp);
 
-    status = lib$spawn(cmd ? &cdsc : NULL, inp ? &idsc : NULL, 
+    status = lib$spawn(cmd ? &cdsc : NULL, inp ? &idsc : NULL,
 	    log ? &ldsc : NULL, 0, 0, 0, &substatus, 0, 0, 0);
     if (status != SS$_NORMAL)
 	substatus = status;
@@ -1939,14 +1959,13 @@ mch_expandpath(struct growarray *gap, char_u *path, int flags)
     char	*cp;
 
     vms_match_num = 0;
-    cnt = decc$to_vms((char *)path,vms_wproc,1,0);
-    if(cnt > 0) cnt = vms_match_num;
-    for(i=0;i<cnt;i++) {
+    cnt = decc$to_vms((char *)path, vms_wproc, 1, 0);
+    if (cnt > 0)
+	cnt = vms_match_num;
+    for (i = 0; i < cnt; i++)
+    {
 	if (mch_getperm(vms_fmatch[i]) >= 0) /* add existing file */
-	{
 	    addfile(gap, vms_fmatch[i], flags);
-	}
-
     }
     return cnt;
 }
